package ro.sync.util.editorvars.expander;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import ro.sync.util.editorvars.parser.model.AskEditorVariable;
import ro.sync.util.editorvars.parser.model.EditorVariable;
import ro.sync.util.editorvars.parser.model.LexicalItem;
import ro.sync.util.editorvars.parser.model.LexicalItem.Type;
import ro.sync.util.editorvars.parser.model.PlainText;

/**
 * Utility methods.
 */
public class EditorVariableUtil {
  /**
   * Constructor.
   */
  private EditorVariableUtil() {}
  
  /**
   * Serializes an editor variable.
   * 
   * @param edVar Editor variable.
   * 
   * @return The string representation. For example: ${framework(name)}
   */
  public static String serialize(EditorVariable edVar) {
    StringBuilder stringBuilder = new StringBuilder();
    
    stringBuilder.append("${");
    stringBuilder.append(edVar.getName());
    
    if (edVar.getType() == Type.ASK_EDITOR_VARIABLE) {
      serializeAskEditorVariableParams(edVar, stringBuilder);
    } else {
      List<LexicalItem> params = edVar.getParams();
      if (params != null && !params.isEmpty()) {
        stringBuilder.append("(");
        serializeParams(params, stringBuilder);
        stringBuilder.append(")");
      }
    }
    stringBuilder.append("}");
    
    return stringBuilder.toString();
  }

  /**
   * Serialize parameters.
   * 
   * @param params The list of parameters.
   * @param stringBuilder A string builder to append to.
   */
  private static void serializeParams(List<LexicalItem> params, StringBuilder stringBuilder) {
    for (int i = 0; i < params.size(); i++) {
      LexicalItem li = params.get(i);
      if (i > 0) {
        stringBuilder.append(", ");
      }

      serialize(li, stringBuilder);
    }
  }

  /**
   * Serializes the lexical unit into the string buffer.
   * 
   * @param li Lexical item to serialize.
   * @param stringBuilder Builder to receive the string version.
   */
  private static void serialize(LexicalItem li, StringBuilder stringBuilder) {
    if (li instanceof PlainText) {
      stringBuilder.append(((PlainText) li).getText());
    } else if (li instanceof EditorVariable) {
      stringBuilder.append(serialize((EditorVariable) li));
    }
  }
  

  /**
   * Serializes the parameters of an $ask editor variable.
   * 
   * @param edVar Editor variable.
   * @param stringBuilder Collector for the string version.
   */
  private static void serializeAskEditorVariableParams(EditorVariable edVar, StringBuilder stringBuilder) {
    AskEditorVariable ask = (AskEditorVariable) edVar;
    // Special function.
    stringBuilder.append("(");
    
    stringBuilder.append("'");
    serialize(ask.getMessage(), stringBuilder);
    stringBuilder.append("'");
    
    if (ask.getInputType() != null) {
      stringBuilder.append(", ");
      serialize(ask.getInputType(), stringBuilder);
    }
    
    Map<LexicalItem, LexicalItem> valueEnumeration = ask.getValueEnumeration();
    
    if (valueEnumeration != null && !valueEnumeration.isEmpty()) {
      stringBuilder.append(", ");
      stringBuilder.append("(");
      Set<LexicalItem> keySet = valueEnumeration.keySet();
      for (Iterator<LexicalItem> iterator = keySet.iterator(); iterator.hasNext();) {
        LexicalItem lexicalItem = iterator.next();
        stringBuilder.append("'");
        serialize(lexicalItem, stringBuilder);
        stringBuilder.append("'");
        stringBuilder.append(":");
        stringBuilder.append("'");
        serialize(valueEnumeration.get(lexicalItem), stringBuilder);
        stringBuilder.append("'");
        if (iterator.hasNext()) {
          stringBuilder.append("; ");  
        }
        
      }
      stringBuilder.append(")");
    } else if (ask.hasPendingValue()) {
        stringBuilder.append(", ");
        stringBuilder.append("(");
        
        serialize((LexicalItem) ask.getPendingValueEnumeration(), stringBuilder);
        
        stringBuilder.append(")");
    }
    
    if (ask.getDefaultValue() != null) {
      stringBuilder.append(", ");
      stringBuilder.append("'");
      serialize(ask.getDefaultValue(), stringBuilder);
      stringBuilder.append("'");
    }
    
    if (ask.getAnswerId() != null) {
      stringBuilder.append(", ");
      serialize(ask.getAnswerId(), stringBuilder);
    }
    
    stringBuilder.append(")");
  }
}
